"""DeploymentConfig entity wrapper for infrastructure tests."""

from __future__ import annotations

from noveler.domain.deployment.value_objects import DeploymentConfig

__all__ = ["DeploymentConfig"]
