"""Wrapper module for DDD compliance tests.

This delegates to the actual PathServiceAdapter implementation under src/noveler.
"""

from noveler.infrastructure.adapters.path_service_adapter import *  # noqa: F403
