# {{ title }}

作成日: {{ created_date }}

## 概要

[作品の概要をここに記載]

## 進捗状況

- [ ] 企画・設計
- [ ] プロット作成
- [ ] 設定集作成
- [ ] 執筆開始
- [ ] 第1話完成

## ディレクトリ構成

```
{{ project_name }}/
├── 10_企画/          # 企画書・市場調査
├── 20_プロット/      # マスタープロット・章別プロット
├── 30_設定集/        # キャラクター・世界観設定
├── 40_原稿/          # 各話の原稿
├── 50_管理資料/      # 進捗管理・品質記録
└── 90_アーカイブ/    # 旧バージョン保管
```
